%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   PP 06/15/05                                              %
%   STOCHASTIC SIMULATIONS OF NF-kB PATHWAY                  %
%   Main Program                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear;              %reset all
clc;                %clear comand window
starttime=clock;    %curent time

%#############################################
%###### Specific imulation parameters ########
%#############################################


N=1;                %1,100 - number of cell in the population considered
Pulse_length=5;    %5, 15, 30, 60 - TNF pulse length in minutes (for persistent stimulation put 360)

%#####################################
%###### Simulation time points #######
%#####################################

t000=10*3600;       %10h randomization of initial conditions
t00=15*3600;        %15h equilibrium waiting time
t0=0.0*3600;        %time when TNF is being introduced into the system
tw1=Pulse_length*60;%length of TNF stimulation
te1=6*3600;         %length of simulation from introducing TNF 
tw2=0*60;           %length of TNF stimulation
te2=0*3600;         %length of simulation from introducing TNF 
tw3=0*60;           %length of TNF stimulation
te3=0*60;           %length of simulation from introducing TNF 

tt=25*3600;         %time up to which the switching time distribution is derived while waiting for equillibrium
tt1=10*3600;        %time up to which the switching time distribution is derived while TNF is on
                    %IMPORTANT -- if theses times are too small, it may result in numerical errors

ANa=2;AN=2;         %Ana - # IKBa alleles, AN # A20 alleles            

timeofhist=[0,2,5,10,15,20,30,60,90,120,180,240,360]*60; %time points for which the data for the histograms are to be collacted
timeofscatter=[0,15,30,90,150,360]*60;                   %time points for which the data for the scatter plots are to be collected 

YYY=0;              %matrix of average, all variables y0(i)(t) 
GGa=0;GG=0;         %average status(t) of Ikba and A20
Bb=0;               %average number of active receptors
PERIOD=[];AMPL=[];NF=[];CYTO=[]; RECN=[]; % saves: 6 (periods and amplitudes) * N cell, NF-kB and cytoplasm volume in N cell,   
zzz=size(timeofscatter);
scatternumb=zzz(2);
YS=zeros(scatternumb,5,N); %1-time points (6), 2-number variables stored (5 variable), 3-number of cells

for i=1:N           %beginning the mean loop 

    %#################################
    %###### Initial conditions #######
    %#################################

    i                   %cell nummber
    rand('state',sum(100*clock))

    TNF=10;             %TNF dose, doses: Allan=20, Lee=10, Hoffmann=10 or 1,  
    y0=zeros(1,15);     %initial conditions set to zero and next: 

    y0(14)=10^5;        %NF-kB is given in cytoplasmic complex(IkBa|NFkB) (total NF-kB kept constant), standard =10^5
    y0(2)=2*10^5;       %initial IKKn, total IKK kept constant  
    y0(11)=0.14*y0(13); %free cytoplasmic IkBa protein 
    y0(12)=0.06*y0(13); %free nuclear IkBa protein
    y0(13)=10;          %IkBa mRNA
    y0(10)=10;          %10 A20 mRNA
    y0(9)=10000;        %10000 A20 protein

    clear V0 V01;
    V01=2*10^(-12);     %randomization of cytoplasmic volume

    clear M M1;
    M1=1000;            %number of receptors

    [k4,ka20,AB,kAB,kv,q1,q2,c1,c3,c4,c5,k1,k2,k3,a1,a2,a3,c1a,c5a,c6a,i1,i1a,e1a,e2a,dt,tp,KN,KNN,ka,ki,kb,kf]=NFparameters(V01,M1);

    y0(10)=AB*y0(10);                   
    y0(9)=AB*y0(9); 

    NF=[NF,y0(14)];     % random intial NF-kB builds vector for all cells 
    CYTO=[CYTO,V01];    % random cytoplasmic volume
    RECN=[RECN,M1];     % random receptor number
    TR=0*TNF;           % TNF off
    Ga=0;               % status of IkBa promoter
    G=0;                % status of A20 promoter
    B=0;                % number of active receptors
    yy0=y0;             % initial conditions y0(i)   

    %###################################################################
    %###### minus one step - randomization of initial condition  #######
    %###################################################################

    realtime=0;                     %simulated time   
    phase=round(rand*t000/dt)*dt;   %randon initial time (dt -simulation time step -10s)  
    tspan=[0:dt:tt];                %length of time for which the solution is derived to find the switching time, tt=25h
   
    while (realtime<phase)
        [T0,Y0]=ode23tb(@NFmodel,tspan,yy0,[],Ga,G,B,V01,M1); 
        Yact=Y0(:,8);               %amount of NF-kBn  
        Yin=Y0(:,12);               %amount of IkBan  
        Gax=Ga;Gx=G;Bx=B;
        [mk,Ga,G,B]=statuschange(AN,ANa,TR,Gax,Gx,Bx,Yact,Yin,V01,M1);  %function determining the change of gene status, call statuschange
        tc=T0(mk);                  %time when the status changes 
        yy0=Y0(mk,:);               %transfer of initial conditions to the next iteration  
        realtime=realtime+tc;
    end;

    if (realtime>phase)
        nn=(realtime-phase)/dt;
        yy0=Y0(mk-nn,:);
        Ga=Gax;G=Gx;B=Bx;
    end;                            %status before the last change it occured ouside of the time interval

    clear Yact Yin Y0 T0 nn mk phase tc;

    %####################################################
    %###### zero step - waiting for "equilibrium" #######
    %####################################################

    realtime=0;
    
    while (realtime<t00)
        [T0,Y0]=ode23tb(@NFmodel,tspan,yy0,[],Ga,G,B,V01,M1); 
        Yact=Y0(:,8);               %amount of NF-kBn  
        Yin=Y0(:,12);               %amount of IkBan  
        Gax=Ga;Gx=G;Bx=B;
        [mk,Ga,G,B]=statuschange(AN,ANa,TR,Gax,Gx,Bx,Yact,Yin,V01,M1); %function determining the change of gene status, call statuschange
        tc=T0(mk);                  %time when the status changes 
        yy0=Y0(mk,:);               %transfer of initial conditions to the next iteration
        realtime=realtime+tc;
    end;

    if (realtime>t00)
        nn=(realtime-t00)/dt;
        yy0=Y0(mk-nn,:);
        Ga=Gax;G=Gx;B=Bx;
    end;                            %status before the last change it occured ouside of the time interval

    clear Yact Yin Y0 T0 nn mk tc; 

    %#######################################
    %###### first step- no TNF still #######
    %#######################################

    realtime=0;
    ga=[Ga];g=[G];                  %saves activity of IkBa and A20 genes 
    bb=[B];                         %saves number of active receptors
    Y=yy0;                          %variables where single cell run is stored
    T=zeros(1,1);
    
    while (realtime<t0)
        [T0,Y0]=ode23tb(@NFmodel,tspan,yy0,[],Ga,G,B,V01,M1); 
        Yact=Y0(:,8);               %amount of NF-kBn  
        Yin=Y0(:,12);               %amount of IkBan  
        Gax=Ga;Gx=G;Bx=B;
        [mk,Ga,G,B]=statuschange(AN,ANa,TR,Gax,Gx,Bx,Yact,Yin,V01,M1); %function determining the change of gene status, call statuschange
        tc=T0(mk);                  %time when the status changes 
        yy0=Y0(mk,:);               %transfer of initial conditions to the next iteration
        Y=[Y;Y0(2:mk,:)];           %rows from 2 do mk, all columns
        T=[T;T0(2:mk)+realtime];          
        ga=[ga;Gax*ones(mk-1,1)];
        g=[g;Gx*ones(mk-1,1)];
        bb=[bb;Bx*ones(mk-1,1)];
        realtime=realtime+tc;
    end;

    if (realtime>t0)
        nn=(realtime-t0)/dt;
        x=size(Y);
        Y=Y(1:(x(1)-nn),:);
        T=T(1:(x(1)-nn));
        yy0=Y0(mk-nn,:);
        x1=length(ga);
        ga=ga(1:x1-nn);g=g(1:x1-nn);bb=bb(1:x1-nn);
        Ga=Gax;G=Gx;B=Bx;
    end;

    %##################################
    %###### second step- TNF on #######
    %##################################

    TR=1*TNF;
    tspan1=[0:dt:tt1];              %length of time for which the solution is derived to find the switching time
    realtime=t0;

    while (realtime<t0+tw1)
        [T0,Y0]=ode23tb(@NFmodel,tspan1,yy0,[],Ga,G,B,V01,M1); 
        Yact=Y0(:,8);                %amount of NF-kBn  
        Yin=Y0(:,12);                %amount of IkBan  
        Gax=Ga;Gx=G;Bx=B;
        [mk,Ga,G,B]=statuschange(AN,ANa,TR,Gax,Gx,Bx,Yact,Yin,V01,M1); %function determining the change of gene status, call statuschange
        tc=T0(mk);                   %time when the status changes 
        yy0=Y0(mk,:);                %transfer of initial conditions to the next iteration
        Y=[Y;Y0(2:mk,:)];                 
        T=[T;T0(2:mk)+realtime];
        ga=[ga;Gax*ones(mk-1,1)];
        g=[g;Gx*ones(mk-1,1)];
        bb=[bb;Bx*ones(mk-1,1)];
        realtime=realtime+tc;
    end;
    
    if (realtime>t0+tw1)
        nn=(realtime-t0-tw1)/dt;
        x=size(Y);
        Y=Y(1:(x(1)-nn),:);
        T=T(1:(x(1)-nn));
        yy0=Y0(mk-nn,:);
        x1=length(ga);
        ga=ga(1:x1-nn);g=g(1:x1-nn);bb=bb(1:x1-nn);
        Ga=Gax;G=Gx;B=Bx;
    end;

    %########################################
    %###### third step-TNF washed out #######
    %########################################

    TR=0*TNF;
    realtime=t0+tw1;
    
    while (realtime<t0+te1)
        [T0,Y0]=ode23tb(@NFmodel,tspan,yy0,[],Ga,G,B,V01,M1); 
        Yact=Y0(:,8);                %amount of NF-kBn  
        Yin=Y0(:,12);                %amount of IkBan  
        Gax=Ga;Gx=G;Bx=B;
        [mk,Ga,G,B]=statuschange(AN,ANa,TR,Gax,Gx,Bx,Yact,Yin,V01,M1); %function determining the change of gene status, call statuschange
        tc=T0(mk);                   %time when the status changes 
        yy0=Y0(mk,:);                %transfer of initial conditions to the next iteration
        Y=[Y;Y0(2:mk,:)];                 
        T=[T;T0(2:mk)+realtime];
        ga=[ga;Gax*ones(mk-1,1)];
        g=[g;Gx*ones(mk-1,1)];
        bb=[bb;Bx*ones(mk-1,1)];
        realtime=realtime+tc;
    end;
    
    if (realtime>t0+te1)
        nn=(realtime-t0-te1)/dt;
        x=size(Y);
        Y=Y(1:(x(1)-nn),:);
        T=T(1:(x(1)-nn));
        yy0=Y0(mk-nn,:);
        x1=length(ga);
        ga=ga(1:x1-nn);g=g(1:x1-nn);bb=bb(1:x1-nn);
        Ga=Gax;G=Gx;B=Bx;
    end;

    %#################################
    %###### TNF on, second time ######
    %#################################

    TR=1*TNF;
    tspan1=[0:dt:tt1];               %length of time for which the solution is derived to find the switching time
    realtime=t0+te1;

    while (realtime<t0+te1+tw2)
        [T0,Y0]=ode23tb(@NFmodel,tspan1,yy0,[],Ga,G,B,V01,M1); 
        Yact=Y0(:,8);                %amount of NF-kBn  
        Yin=Y0(:,12);                %amount of IkBan  
        Gax=Ga;Gx=G;Bx=B;
        [mk,Ga,G,B]=statuschange(AN,ANa,TR,Gax,Gx,Bx,Yact,Yin,V01,M1); %function determining the change of gene status, call statuschange
        tc=T0(mk);                   %time when the status changes 
        yy0=Y0(mk,:);                %transfer of initial conditions to the next iteration
        Y=[Y;Y0(2:mk,:)];                 
        T=[T;T0(2:mk)+realtime];
        ga=[ga;Gax*ones(mk-1,1)];
        g=[g;Gx*ones(mk-1,1)];
        bb=[bb;Bx*ones(mk-1,1)];
        realtime=realtime+tc;
    end;
    
    if (realtime>t0+te1+tw2)
        nn=(realtime-t0-te1-tw2)/dt;
        x=size(Y);
        Y=Y(1:(x(1)-nn),:);
        T=T(1:(x(1)-nn));
        yy0=Y0(mk-nn,:);
        x1=length(ga);
        ga=ga(1:x1-nn);g=g(1:x1-nn);bb=bb(1:x1-nn);
        Ga=Gax;G=Gx;B=Bx;
    end;

    %##########################################
    %###### TNF washed out  second time #######
    %##########################################

    TR=0*TNF;
    realtime=t0+te1+tw2;
   
    while (realtime<t0+te1+te2)
        [T0,Y0]=ode23tb(@NFmodel,tspan,yy0,[],Ga,G,B,V01,M1); 
        Yact=Y0(:,8);                %amount of NF-kBn  
        Yin=Y0(:,12);                %amount of IkBan  
        Gax=Ga;Gx=G;Bx=B;
        [mk,Ga,G,B]=statuschange(AN,ANa,TR,Gax,Gx,Bx,Yact,Yin,V01,M1); %function determining the change of gene status, call statuschange
        tc=T0(mk);                   %time when the status changes 
        yy0=Y0(mk,:);                %transfer of initial conditions to the next iteration
        Y=[Y;Y0(2:mk,:)];                 
        T=[T;T0(2:mk)+realtime];
        ga=[ga;Gax*ones(mk-1,1)];
        g=[g;Gx*ones(mk-1,1)];
        bb=[bb;Bx*ones(mk-1,1)];
        realtime=realtime+tc;
    end;
    
    if (realtime>t0+te1+te2)
        nn=(realtime-t0-te1-te2)/dt;
        x=size(Y);
        Y=Y(1:(x(1)-nn),:);
        T=T(1:(x(1)-nn));
        yy0=Y0(mk-nn,:);
        x1=length(ga);
        ga=ga(1:x1-nn);g=g(1:x1-nn);bb=bb(1:x1-nn);
        Ga=Gax;G=Gx;B=Bx;
    end;

    %#################################
    %###### TNF on, third time #######
    %#################################

    TR=1*TNF;
    tspan1=[0:dt:tt1];               %length of time for which the solution is derived to find the switching time
    realtime=t0+te1+te2;

    while (realtime<t0+te1+te2+tw3)
        [T0,Y0]=ode23tb(@NFmodel,tspan1,yy0,[],Ga,G,B,V01,M1); 
        Yact=Y0(:,8);                %amount of NF-kBn  
        Yin=Y0(:,12);                %amount of IkBan  
        Gax=Ga;Gx=G;Bx=B;
        [mk,Ga,G,B]=statuschange(AN,ANa,TR,Gax,Gx,Bx,Yact,Yin,V01,M1); %function determining the change of gene status, call statuschange
        tc=T0(mk);                   %time when the status changes 
        yy0=Y0(mk,:);                %transfer of initial conditions to the next iteration
        Y=[Y;Y0(2:mk,:)];                 
        T=[T;T0(2:mk)+realtime];
        ga=[ga;Gax*ones(mk-1,1)];
        g=[g;Gx*ones(mk-1,1)];
        bb=[bb;Bx*ones(mk-1,1)];
        realtime=realtime+tc;
    end;
    
    if (realtime>t0+te1+te2+tw3)
        nn=(realtime-t0-te1-te2-tw3)/dt;
        x=size(Y);
        Y=Y(1:(x(1)-nn),:);
        T=T(1:(x(1)-nn));
        yy0=Y0(mk-nn,:);
        x1=length(ga);
        ga=ga(1:x1-nn);g=g(1:x1-nn);bb=bb(1:x1-nn);
        Ga=Gax;G=Gx;B=Bx;
    end;

    %#########################################
    %###### TNF washed out  third time #######
    %#########################################

    TR=0*TNF;
    realtime=t0+te1+te2+tw3;
    
    while (realtime<t0+te1+te2+te3)
        [T0,Y0]=ode23tb(@NFmodel,tspan,yy0,[],Ga,G,B,V01,M1); 
        Yact=Y0(:,8);                %amount of NF-kBn  
        Yin=Y0(:,12);                %amount of IkBan  
        Gax=Ga;Gx=G;Bx=B;
        [mk,Ga,G,B]=statuschange(AN,ANa,TR,Gax,Gx,Bx,Yact,Yin,V01,M1); %function determining the change of gene status, call statuschange
        tc=T0(mk);                   %time when the status changes 
        yy0=Y0(mk,:);                %transfer of initial conditions to the next iteration
        Y=[Y;Y0(2:mk,:)];                 
        T=[T;T0(2:mk)+realtime];
        ga=[ga;Gax*ones(mk-1,1)];
        g=[g;Gx*ones(mk-1,1)];
        bb=[bb;Bx*ones(mk-1,1)];
        realtime=realtime+tc;
    end;
    
    if (realtime>t0+te1+te2+te3)
        nn=(realtime-t0-te1-te2-te3)/dt;
        x=size(Y);
        Y=Y(1:(x(1)-nn),:);
        T=T(1:(x(1)-nn));
        yy0=Y0(mk-nn,:);
        x1=length(ga);
        ga=ga(1:x1-nn);g=g(1:x1-nn);bb=bb(1:x1-nn);
        Ga=Gax;G=Gx;B=Bx;
    end;

    YYY=YYY+Y;
    GGa=GGa+ga;
    GG=GG+g;
    Bb=Bb+bb;
end;                             %end of the Main LOOP

clear tspan tspan1 g ga T0 Y0 Yact Yin Y y0 yy0 tindex;

%####################
%###### PLOTS #######
%####################

YYY=YYY/N;                       %average over population
GGa=GGa/N;
GG=GG/N;
Bb=Bb/N;
T=T/3600;

if (N==1)
    GGa=GGa+0.001;               %for nice pictures    
    GG=GG+0.001;
end;

simulation_time=etime(clock,starttime)/3600 %simulation time in hours

averageCellPlotting
